package com.reactnativevidvocr;

import android.app.Activity;
import android.graphics.Color;
import androidx.annotation.NonNull;

import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.google.gson.Gson;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import me.vidv.vidvocrsdk.sdk.BuilderError;
import me.vidv.vidvocrsdk.sdk.CapturedImages;
import me.vidv.vidvocrsdk.sdk.ServiceFailure;
import me.vidv.vidvocrsdk.sdk.Success;
import me.vidv.vidvocrsdk.sdk.UserExit;
import me.vidv.vidvocrsdk.sdk.VIDVOCRConfig;
import me.vidv.vidvocrsdk.sdk.VIDVOCRListener;
import me.vidv.vidvocrsdk.sdk.VIDVOCRResponse;
import me.vidv.vidvocrsdk.viewmodel.CapturedImageData;
import me.vidv.vidvocrsdk.viewmodel.VIDVError;
import me.vidv.vidvocrsdk.viewmodel.VIDVEvent;
import me.vidv.vidvocrsdk.viewmodel.VIDVLogListener;

@ReactModule(name = VidvocrModule.NAME)
public class VidvocrModule extends ReactContextBaseJavaModule {
    public static final String NAME = "Vidvocr";
 private Promise promise;
  private VIDVOCRConfig.Builder builder;
  private String PluginInitialzingError = "6001";
  private String jsonInString;
  private ArrayList<CapturedImageData> capturedImages = new ArrayList<>();
 private ArrayList<VIDVError> errors = new ArrayList<>();
    private ArrayList<VIDVEvent> events = new ArrayList<>();

    public VidvocrModule(ReactApplicationContext reactContext) {
        super(reactContext);
         capturedImages=new ArrayList<>();
           errors = new ArrayList<>();
        events = new ArrayList<>();
         builder = new VIDVOCRConfig.Builder();
    }

    @Override
    @NonNull
    public String getName() {
        return NAME;
    }


    @ReactMethod
  public void startOCR(ReadableMap params, Promise promise) {
    this.promise = promise;
    capturedImages=new ArrayList<>();
    errors = new ArrayList<>();
    events = new ArrayList<>();
    builder = new VIDVOCRConfig.Builder();
    Activity activity = getCurrentActivity();
    HashMap<String, Object> inputs = params.toHashMap();

    if (!inputs.containsKey("base_url")) {
      promise.reject(PluginInitialzingError, "please make sure you entered base url correctly");
    } else if (!inputs.containsKey("access_token")) {
      promise.reject(PluginInitialzingError, "please make sure you entered access token correctly");
    } else if (!inputs.containsKey("bundle_key")) {
      promise.reject(PluginInitialzingError, "please make sure you entered bundle key correctly");
    } else {
      if (!inputs.containsKey("language") || inputs.get("language").toString().equals(""))
        builder.setLanguage("en");
      else builder.setLanguage(inputs.get("language").toString());

      if (inputs.containsKey("document_verification"))
        builder.setDocumentVerification((Boolean) inputs.get("document_verification"));

      if (inputs.containsKey("data_validation"))
        builder.setDataValidation((Boolean) inputs.get("data_validation"));

      if (inputs.containsKey("return_data_validation_error"))
        builder.setReturnValidationError((Boolean) inputs.get("return_data_validation_error"));

      if (inputs.containsKey("review_data"))
        builder.setReviewData((Boolean) inputs.get("review_data"));

      if (inputs.containsKey("capture_only_mode"))
        builder.setCaptureOnlyMode((Boolean) inputs.get("capture_only_mode"));
      
      if (inputs.containsKey("manual_capture_mode"))
        builder.setManualCaptureMode((Boolean) inputs.get("manual_capture_mode"));

      if (inputs.containsKey("preview_captured_image"))
        builder.setPreviewCapturedImage((Boolean) inputs.get("preview_captured_image"));

      if (inputs.containsKey("headers")) {
        HashMap<String, Object> map = (HashMap<String, Object>) inputs.get("headers");
        if (map.size() > 0) {
          HashMap<String, String> map2 = new HashMap<>();
          for (Map.Entry<String, Object> header : map.entrySet()) {
            map2.put(header.getKey(), header.getValue().toString());
          }
          builder.setHeaders(map2);
        }
      }

      if (inputs.containsKey("ssl_certificate")) {
        builder.setSSLCertificate((InputStream) inputs.get("ssl_certificate"));
      }
      if (inputs.containsKey("primary_color")&& !inputs.get("primary_color").toString().equals("")) {
        builder.setPrimaryColor(Color.parseColor((String) inputs.get("primary_color")));
      }

      builder.setBaseUrl(params.getString("base_url"))
        .setAccessToken(params.getString("access_token"))
        .setBundleKey(params.getString("bundle_key"));

      if (inputs.containsKey("enable_logging")&& (Boolean) inputs.get("enable_logging") ) {

                builder.setLogsListener(new VIDVLogListener() {
                    @Override
                    public void onLog(VIDVEvent log) {
                        events.add(log);
                    }

                    @Override
                    public void onLog(VIDVError log) {
                        errors.add(log);
                    }
                });
            }

      builder.start(activity, new VIDVOCRListener() {
        @Override
        public void onOCRResult(VIDVOCRResponse response) {
          if (response instanceof Success) {
            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "SUCCESS");
              jsonObject.put("ocrResult", ((Success) response).getData());
              jsonObject.put("capturedImages", capturedImages);
                if (inputs.containsKey("enable_logging")&& (Boolean) inputs.get("enable_logging") ) {
                                jsonObject.put("events", events);
                                jsonObject.put("errors", errors);
                            }
            } catch (JSONException e) {
              e.printStackTrace();
            }
            jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof BuilderError) {
            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "ERROR");
              jsonObject.put("errorCode", ((BuilderError) response).getCode());
              jsonObject.put("errorMessage", ((BuilderError) response).getMessage());
           
            } catch (JSONException e) {
              e.printStackTrace();
            }
            jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof ServiceFailure) {
            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "FAILURE");
              jsonObject.put("errorCode", ((ServiceFailure) response).getCode());
              jsonObject.put("errorMessage", ((ServiceFailure) response).getMessage());
              jsonObject.put("ocrResult", ((ServiceFailure) response).getData());
              jsonObject.put("capturedImages", capturedImages);
                if (inputs.containsKey("enable_logging")&& (Boolean) inputs.get("enable_logging") ) {
                                jsonObject.put("events", events);
                                jsonObject.put("errors", errors);
                            }
            } catch (JSONException e) {
              e.printStackTrace();
            }
            jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof UserExit) {
            JSONObject jsonObject = new JSONObject();
            try {
              jsonObject.put("state", "EXIT");
              jsonObject.put("step", ((UserExit) response).getStep());
              jsonObject.put("ocrResult", ((UserExit) response).getData());
              jsonObject.put("capturedImages", capturedImages);
                if (inputs.containsKey("enable_logging")&& (Boolean) inputs.get("enable_logging") ) {
                                jsonObject.put("events", events);
                                jsonObject.put("errors", errors);
                            }
            } catch (JSONException e) {
              e.printStackTrace();
            }
            jsonInString = new Gson().toJson(jsonObject);
            promise.resolve(jsonInString);
          } else if (response instanceof CapturedImages) {
            capturedImages.add(((CapturedImages) response).getImages());
          }
        }
      });
    }
  }
}
